import pathlib
import tempfile
import unittest
import torch
from irisml.tasks.save_tensor_list_jsonl import Task


class TestSaveTensorListJsonl(unittest.TestCase):
    def test_simple(self):
        tensors = [torch.tensor([1, 2, 3]), torch.tensor([4, 5, 6])]
        with tempfile.TemporaryDirectory() as tempdir:
            path = pathlib.Path(tempdir) / 'test.jsonl'
            Task(Task.Config(path=path)).execute(Task.Inputs(data=tensors))
            lines = path.read_text().splitlines()
            self.assertEqual(len(lines), 2)
            self.assertEqual(lines[0], '[1, 2, 3]')
            self.assertEqual(lines[1], '[4, 5, 6]')



import io
import unittest

import torch
from irisml.tasks.average_state_dict import Task


class TestAverageStateDict(unittest.TestCase):
    def test_state_dicts(self):
        input_state_dicts = {'a': torch.rand(1), 'b': torch.rand((2, 3))}
        input_state_dicts2 = {'a': torch.rand(1), 'b': torch.rand((2, 3))}
        input_state_dicts3 = {'a': torch.rand(1), 'b': torch.rand((2, 3))}

        averaged_state_dict = {'a': (input_state_dicts['a'] + input_state_dicts2['a'] + input_state_dicts3['a']) / 3,
                               'b': (input_state_dicts['b'] + input_state_dicts2['b'] + input_state_dicts3['b']) / 3}
        output_state_dict = Task(Task.Config()).execute(Task.Inputs(state_dict_list=[input_state_dicts, input_state_dicts2, input_state_dicts3])).state_dict

        self.assertEqual(set(output_state_dict.keys()), {'a', 'b'})
        self.assertTrue(torch.allclose(output_state_dict['a'], averaged_state_dict['a']))
        self.assertTrue(torch.allclose(output_state_dict['b'], averaged_state_dict['b']))

        input_state_dict_bytes = self._serialize_state_dict(input_state_dicts)
        input_state_dict_bytes2 = self._serialize_state_dict(input_state_dicts2)
        input_state_dict_bytes3 = self._serialize_state_dict(input_state_dicts3)

        output_state_dict = Task(Task.Config()).execute(Task.Inputs(state_dict_bytes_list=[input_state_dict_bytes, input_state_dict_bytes2, input_state_dict_bytes3])).state_dict
        self.assertEqual(set(output_state_dict.keys()), {'a', 'b'})
        self.assertTrue(torch.allclose(output_state_dict['a'], averaged_state_dict['a']))
        self.assertTrue(torch.allclose(output_state_dict['b'], averaged_state_dict['b']))

    def test_wrong_key_set(self):
        input_state_dicts = {'a': torch.rand(1), 'b': torch.rand((2, 3))}
        missing_key = {'b': torch.rand((2, 3))}

        with self.assertRaises(ValueError):
            Task(Task.Config()).execute(Task.Inputs(state_dict_list=[input_state_dicts, missing_key]))

        extra_key = {'a': torch.rand(1), 'b': torch.rand((2, 3)), 'c': torch.rand(1)}
        with self.assertRaises(ValueError):
            Task(Task.Config()).execute(Task.Inputs(state_dict_list=[input_state_dicts, extra_key]))

    @staticmethod
    def _serialize_state_dict(state_dict):
        buffer = io.BytesIO()
        torch.save(state_dict, buffer)
        return buffer.getbuffer().tobytes()



import unittest
from irisml.tasks.make_list import Task


class TestMakeList(unittest.TestCase):
    def test_dict(self):
        dict0 = {'a': 1}
        dict1 = {'b': 2}
        dict2 = {'c': 3}

        outputs = Task(Task.Config()).execute(Task.Inputs(input_dict0=dict0, input_dict1=dict1, input_dict2=dict2))
        self.assertEqual(outputs.list_dict, [dict0, dict1, dict2])

    def test_bytes(self):
        bytes0 = b'abc'
        bytes1 = b'def'
        bytes2 = b'ghi'

        outputs = Task(Task.Config()).execute(Task.Inputs(input_bytes0=bytes0, input_bytes1=bytes1, input_bytes2=bytes2))
        self.assertEqual(outputs.list_bytes, [bytes0, bytes1, bytes2])

    def test_multiple_inputs(self):
        with self.assertRaises(ValueError):
            Task(Task.Config()).execute(Task.Inputs(input_dict0={'a': 3}, input_bytes0=b''))

    def test_empty(self):
        with self.assertRaises(ValueError):
            Task(Task.Config()).execute(Task.Inputs())



import unittest
import torch
from irisml.tasks.normalize_tensor_last_dim import Task


class TestNormalizeTensorLastDim(unittest.TestCase):
    def test_simple(self):
        tensor = torch.rand((2, 3, 4))
        outputs = Task(Task.Config()).execute(Task.Inputs(tensor=tensor))

        self.assertIsInstance(outputs.tensor, torch.Tensor)
        self.assertEqual(outputs.tensor.shape, tensor.shape)
        self.assertTrue(torch.allclose(outputs.tensor.norm(dim=-1), torch.ones((2, 3))))

    def test_zero(self):
        """Test that the task doesn't crash when the input tensor is all zeros."""
        tensor = torch.zeros((2, 3, 4))
        outputs = Task(Task.Config()).execute(Task.Inputs(tensor=tensor))

        self.assertIsInstance(outputs.tensor, torch.Tensor)
        self.assertEqual(outputs.tensor.shape, tensor.shape)

    def test_scalar(self):
        tensor = torch.tensor(3.0)
        outputs = Task(Task.Config()).execute(Task.Inputs(tensor=tensor))

        self.assertIsInstance(outputs.tensor, torch.Tensor)
        self.assertEqual(outputs.tensor.shape, tensor.shape)
        self.assertAlmostEqual(outputs.tensor.item(), 1.0)



import dataclasses
import logging
import irisml.core

logger = logging.getLogger(__name__)


class Task(irisml.core.TaskBase):
    """Create a list from input objects.

    This is a work around for the lack of support for list inputs in IrisML.
    """
    VERSION = '0.1.0'

    @dataclasses.dataclass
    class Inputs:
        input_dict0: dict | None = None
        input_dict1: dict | None = None
        input_dict2: dict | None = None
        input_dict3: dict | None = None
        input_dict4: dict | None = None
        input_bytes0: bytes | None = None
        input_bytes1: bytes | None = None
        input_bytes2: bytes | None = None
        input_bytes3: bytes | None = None
        input_bytes4: bytes | None = None

    @dataclasses.dataclass
    class Outputs:
        list_dict: list[dict] | None = None
        list_bytes: list[bytes] | None = None

    def execute(self, inputs):
        input_dicts = [inputs.input_dict0, inputs.input_dict1, inputs.input_dict2, inputs.input_dict3, inputs.input_dict4]
        input_bytes = [inputs.input_bytes0, inputs.input_bytes1, inputs.input_bytes2, inputs.input_bytes3, inputs.input_bytes4]

        if any(d is not None for d in input_dicts) and any(b is not None for b in input_bytes):
            raise ValueError("Only one type of input can be provided")

        result = [d for d in input_dicts if d is not None]
        if result:
            logger.info(f"Created a list of dictionaries with {len(result)} elements")
            return self.Outputs(list_dict=result)

        result = [b for b in input_bytes if b is not None]
        if result:
            logger.info(f"Created a list of bytes with {len(result)} elements")
            return self.Outputs(list_bytes=result)

        raise ValueError("At least one input must be provided")

    def dry_run(self, inputs):
        return self.execute(inputs)
